import { getTempFileURL } from '../../../utils/tcb';
import equal from '../../../utils/deepEqual';
import { upload, handleUpload, getName, initUploadInstance } from './upload';
Component({
  options: {
    virtualHost: true,
    multipleSlots: true,
  },
  properties: {
    label: {
      type: String,
      value: '文件上传',
    },
    //是否显示标题
    labelVisible: {
      type: Boolean,
      value: true,
    },
    //单文件上传模式
    single: {
      type: Boolean,
      value: false,
    },
    defaultValue: {
      type: null,
    },
    oldValue: {
      type: null,
    },
    maxUploadCount: {
      type: Number,
      value: 9,
    },
    //单个文件最大使用限制
    maxSize: {
      type: Number,
      value: 10,
    },
    layout: {
      type: String,
      value: '',
    },
    extClass: {
      type: String,
      value: '',
    },
    tips: {
      type: Boolean,
      value: false,
    },
    status: {
      type: String,
      valve: 'success',
    },
    downloadVisible: {
      type: Boolean,
      value: true,
    },
    requiredFlag: {
      type: Boolean,
      value: false,
    },
    disabled: {
      type: Boolean,
      value: false,
    },
    readOnly: {
      type: Boolean,
      value: false,
    },
    callbacks: {
      type: Object,
      value: null,
    },
    uploadTipText: {
      type: String,
      value: '支持批量上传',
    },
    uploadButtonText: {
      type: String,
      value: '点击上传',
    },
  },
  data: {
    maxCount: 0,
    files: [],
    urls: [],
    cloudFile: [],
    strogeUrls: [],
    singleFlag: false,

    layoutCls: '',
    showActionsheet: false,
    groups: [
      { text: '照片图库', value: 1 },
      { text: '拍照或录像', value: 2 },
      { text: '选取聊天文件', value: 3 },
    ],
    uploadInstance: {},
  },
  methods: {
    onDelete: function (e) {
      let newUrls = [];
      if (this.data.urls.length == 0) {
        this.data.cloudFile.forEach((v) => {
          if (e.target.dataset.item.cloudPath != v) {
            newUrls.push(v);
          }
        });
        this.setData({
          files: this.data.files.filter((v) => e.target.dataset.item.cloudPath !== v.cloudPath),
          urls: newUrls,
        });
      } else {
        const filelist = this.data.files.filter(
          (item) =>
            e.target.dataset.item?.path != item?.path || e.target.dataset.item?.tempFilePath != item?.tempFilePath,
        );
        newUrls = this.data.urls.filter((v) => e.target.dataset.item?.cloudPath != v);
        this.setData({
          files: filelist,
          urls: newUrls,
        });
      }
      this.handleChange(newUrls);
    },

    // change 事件
    handleChange: function (value) {
      this.setData({ oldValue: value });
      this.triggerEvent('change', { value });
    },

    reload: function (e) {
      const { uploadInstance } = this.data;
      handleUpload([e.target.dataset.item], uploadInstance, true);
    },
    downloadFile: async function (e) {
      const path = e.target.dataset.item?.cloudPath;

      const fileSrc = /^cloud:/.test(path) ? await getTempFileURL(path) : path;
      if (fileSrc) {
        wx.showLoading({
          title: '下载中',
          mask: true,
        });
        wx.downloadFile({
          url: fileSrc,
          success(res) {
            wx.hideLoading();
            const fileManager = wx.getFileSystemManager();
            fileManager.saveFile({
              tempFilePath: res.tempFilePath,
              success: (res) => {
                wx.hideLoading();
                wx.openDocument({
                  filePath: res.savedFilePath,
                  showMenu: true,
                  fail: function () {
                    // https://developers.weixin.qq.com/community/develop/doc/0006a6b6f2cf60404299d51ab56809?highLine=wx.openDocument
                    wx.showToast({
                      title: `该文档类型暂不支持下载`,
                      icon: 'none',
                      duration: 2000,
                    });
                  },
                });
              },
            });
          },
          fail(e) {
            wx.hideLoading();
            wx.showToast({
              title: e.errMsg,
              icon: 'none',
              duration: 2000,
            });
          },
        });
      }
    },
    onButtonTap: function () {
      this.setData({
        showActionsheet: true,
      });
    },
    onActionTap: function (e) {
      this.setData({
        showActionsheet: false,
      });
      const { disabled, maxCount, files, maxSize } = this.data;
      let config = { disabled, count: maxCount, maxSize, action: 'CHOOSE_MEDIA' };

      switch (e?.detail?.value) {
        // 照片图库
        case 1:
          // 拍照或录像
          //从手机相册中选择图片或视频
          config.sourceType = ['album'];
          break;
        case 2:
          // 拍摄
          config.sourceType = ['camera'];
          break;
        // 选取聊天文件
        case 3:
          config.type = 'all';
          config.action = 'CHOOSE_MESSAGE_FILE';
          break;
        default:
          break;
      }
      const uploadInstance = initUploadInstance({ config, previewFile: files }, this);
      this.setData({ uploadInstance });
      upload(uploadInstance);
    },
  },
  observers: {
    defaultValue: async function (value) {
      if (equal(this.data.oldValue, value)) {
        return;
      }
      //当单图片上传且默认值为空数组时，初次渲染更新value，以避免空串提交类型不匹配问题
      if (Array.isArray(value) && !this.data.singleFlag) {
        this.setData({
          urls: value,
        });
        this.data.singleFlag = true;
      }
      let httpFiles = [];
      if (value && JSON.stringify(value) != '[]') {
        const values = Array.isArray(value) ? value : [value];
        for (let f of values) {
          if (f.startsWith('cloud:')) {
            const hf = await getTempFileURL(f);
            const label = getName(hf);
            const httpFile = {
              name: label,
              cloudPath: f,
              filePath: hf,
              path: hf,
              status: 'success',
            };
            httpFiles.push(httpFile);
          }
        }
        this.setData({
          oldValue: value,
          files: httpFiles,
          cloudFile: JSON.stringify(value) != JSON.stringify(this.data.cloudFile) ? [].concat(value) : [],
        });
      } else {
        this.setData({
          oldValue: value,
          files: [],
          cloudFile: [],
        });
      }
    },
    'maxUploadCount, single': function (maxUploadCount, single) {
      const maxCount = single ? 1 : maxUploadCount;
      this.setData({
        maxCount,
      });
    },
    'layout,extClass': function (layout, extClass) {
      const temp = `weda-formcells weui-cells weui-cells_forms ${extClass}`;
      const layoutCls = layout !== 'vertical' ? ` weui-flex ${temp}` : `${temp}`;
      this.setData({ layoutCls });
    },
  },
});
