/* eslint-disable @typescript-eslint/no-magic-numbers */
import handleEvents from '../../utils/handleEvents';
import { commonCompBehavior } from '../../utils/common-behavior';
import { WD_PREFIX } from '../../utils/constant';
import { compareVersion } from '../../utils/platform';
import { errorHandler } from '../../utils/error';

Component({
  options: { virtualHost: true },
  behaviors: [commonCompBehavior],
  properties: {
    className: {
      type: String,
      value: '',
    },
    style: {
      type: String,
      value: '',
    },
    id: {
      type: String,
      value: '',
    },
    adType: {
      type: String,
      value: 'banner',
    },
    unitId: {
      type: String,
    },
    adTheme: {
      type: String,
      value: 'white',
    },
    adIntervals: {
      type: Number,
    },
    gridCount: {
      type: Number,
      value: 5,
    },
    multiton: {
      type: Boolean,
      value: false,
    },
    disableFallbackSharePage: {
      type: Boolean,
      value: false,
    },
  },
  data: {
    _adType: 'banner',
    _gridCount: 5,
    _adIntervals: null,
    classPrefix: WD_PREFIX,
    rewardedVideoAd: null,
  },
  methods: {
    ...handleEvents([
      { name: 'load', title: '广告加载成功' },
      { name: 'error', title: '广告加载失败' },
      { name: 'close', title: '广告被关闭' },
    ]),
    checkSdkVersion() {
      let canUseRewardedVideoAd = true;
      if (wx.canIUse('getAppBaseInfo')) {
        const appBaseInfo = wx.getAppBaseInfo();
        canUseRewardedVideoAd = compareVersion(appBaseInfo.SDKVersion, '2.8.1') >= 0;
      } else {
        const version = wx.getSystemInfoSync().SDKVersion;
        canUseRewardedVideoAd = compareVersion(version, '2.8.1') >= 0;
      }
      return canUseRewardedVideoAd;
    },
    createRewardedVideoAd() {
      if (this.data.rewardedVideoAd) {
        this.destroy();
      }
      const canUseRewardedVideoAd = this.checkSdkVersion();

      if (wx.createRewardedVideoAd && canUseRewardedVideoAd) {
        const { unitId, multiton, disableFallbackSharePage } = this.data;
        const rewardedVideoAd = wx.createRewardedVideoAd({
          adUnitId: unitId,
          multiton,
          disableFallbackSharePage,
        });
        rewardedVideoAd.onLoad(() => {
          this.triggerEvent('load');
        });
        rewardedVideoAd.onError((error) => {
          const { comErrorInfo } = errorHandler({
            id: this.id,
            code: 'WdAd.createRewardedVideoAd',
            error: error,
          });
          this.triggerEvent('error', { error: comErrorInfo });
        });
        rewardedVideoAd.onClose((res) => {
          this.triggerEvent('close', res);
        });
        this.setData({ rewardedVideoAd });
      } else {
        const { comErrorInfo } = errorHandler({
          id: this.id,
          code: 'WdAd.createRewardedVideoAd',
          message: '当前基础库版本低于2.8.1，不支持激励视频广告功能',
        });
        this.triggerEvent('error', { error: comErrorInfo });
      }
    },
    destroy() {
      this.data.rewardedVideoAd?.destroy();
      this.setData({
        rewardedVideoAd: null,
      });
    },
    show() {
      const { rewardedVideoAd } = this.data;
      rewardedVideoAd?.show().catch(() => {
        rewardedVideoAd
          .load()
          .then(() => rewardedVideoAd.show())
          .catch((error) => {
            const { comErrorInfo } = errorHandler({
              id: this.id,
              code: 'WdAd.showRewardedVideoAd',
              error: error,
            });
            this.triggerEvent('error', { error: comErrorInfo });
          });
      });
    },
    updateWidgetAPI() {
      this.setReadonlyAttributes?.({
        rewardedVideoAd: this.data.rewardedVideoAd,
        show: this.show.bind(this),
      });
    },
  },
  observers: {
    // 规范取值
    'adType,adIntervals,gridCount': function (adType, adIntervals, gridCount) {
      const _adType = ['banner', 'video', 'grid', 'rewardedVideoAd'].includes(adType) ? adType : 'banner';
      const _adIntervals = Number(adIntervals) >= 30 ? Number(adIntervals) : null;
      const _gridCount = gridCount == 8 ? 8 : 5;
      this.setData({ _adType, _gridCount, _adIntervals });
    },
    rewardedVideoAd: function () {
      this.updateWidgetAPI();
    },
  },
  lifetimes: {
    attached() {
      if (this.data.adType === 'rewardedVideoAd') {
        this.createRewardedVideoAd();
      }
    },
    detached() {
      this.destroy();
    },
  },
});
