import { commonCompBehavior } from '../../utils/common-behavior';
import formFieldBehavior from '../form-field-behavior/form-field-behavior';
import itemBehavior from '../form-field-behavior/item-behavior';
import { convertFixedIcon, SELECT_ICON_H5 } from '../../utils/getFormLegacy';
import debounce from '../../utils/debounce';
import { arrayToMap, getSelected } from '../../utils/tool';

Component({
  options: { virtualHost: true, styleIsolation: 'shared' },
  behaviors: [itemBehavior, commonCompBehavior, formFieldBehavior],
  properties: {
    classRoot: {
      type: String,
      value: 'select',
    },
    supportManyRelated: {
      type: Boolean,
      value: false,
    },
    queryCondition: {
      type: null,
      value: [],
    },
    sorter: {
      type: Array,
      value: [],
    },
    searchable: {
      type: Boolean,
      value: true,
    },
    filterable: {
      type: Boolean,
      value: false,
    },
  },
  data: {
    itemMap: {},
    selectedLabel: null,
    selectedItem: null,
    _suffixType: '',
    _suffixIcon: '',
  },
  methods: {
    finalUpdateWidgetAPI: function () {
      const { selectedLabel, primaryField, selectedItem } = this.properties;
      this.setReadonlyAttributes &&
        this.setReadonlyAttributes({
          selectedLabel,
          selectFields: [primaryField],
          item: selectedItem,
        });
      this.updateWidgetAPI();
    },
    search: function (e) {
      this.debouncedTriggerSearchEvent(e.detail.value);
    },
    debouncedTriggerSearchEvent: debounce(function (value) {
      this.triggerEvent('search', { value });
    }),
    updateSelect: function (e) {
      const { value, option } = e.detail;
      const itemMap = arrayToMap(option, 'value');
      const { selectedLabel, selectedItem } = getSelected(itemMap, value, false);
      this.setData({ selectedLabel, selectedItem });
    },
  },
  observers: {
    'name, value, label, required, visible, disabled, readOnly, before, after, primaryField, selectedLabel, selectedItem':
      function () {
        this.finalUpdateWidgetAPI();
      },
    'suffixType,suffixIcon': function (suffixType, suffixIcon) {
      const [_suffixType, _suffixIcon] = convertFixedIcon(suffixType, suffixIcon, SELECT_ICON_H5);
      this.setData({ _suffixType, _suffixIcon });
    },
    range: function (range) {
      this.updateSelect({ detail: { value: this.data.value, option: range } });
    },
  },
  lifetimes: {
    attached: function () {
      this.finalUpdateWidgetAPI();
    },
  },
});
