Component({
  properties: {
    isActive: {
      type: Boolean,
      value: true,
    },
    option: {
      type: Array,
      value: [],
    },
    searchStatus: {
      type: Number,
      value: 0,
    },
    loadStatus: {
      type: Number,
      value: 0,
    },
    disabled: {
      type: Boolean,
      value: false,
    },
    ignoreCase: {
      type: Boolean,
      value: true,
    },
    selectedValue: {
      type: Array,
      value: [],
    },
    searchable: {
      type: Boolean,
      value: true,
    },
    filterable: {
      type: Boolean,
      value: false,
    },
    multiple: {
      type: Boolean,
      value: false,
    },
    currentPageNo: {
      type: Number,
      value: 1,
    },
  },
  data: {
    height: '390px',
    focus: false,
    searchValue: '',
    index: -1,
    status: 0,
    showOption: [],
    pageNo: 1,
    selectedCache: [], // 缓存当前选中值
  },

  methods: {
    cancelPicker: function (e) {
      if (this.data.multiple) {
        const { option } = this.properties;
        this.setData({ selectedCache: option.filter((item) => item.check) });
      }
      this.triggerEvent('onClosePicker');
    },
    // 监听输入
    bindinput: function (event) {
      const value = event.detail.value;
      this.setData({
        searchValue: value,
        pageNo: 1,
      });
      this.triggerEvent('search', { value });
    },
    // 清空输入
    clear: function () {
      this.setData({
        searchValue: '',
        showOption: this.properties.option,
        pageNo: 1,
      });
      this.triggerEvent('search', { value: '' });
    },
    // 取消
    shrink: function () {
      this.setData({
        height: '390px',
        focus: false,
        searchValue: '',
        pageNo: 1,
        showOption: this.properties.option,
      });
      this.triggerEvent('search', { value: '', noEvent: true });
    },
    confirmModal: function () {
      const { selectedCache } = this.data;
      this.triggerEvent('onSelectPicker', selectedCache);
    },
    // 获取焦点
    onFocus: function () {
      this.setData({
        height: '550px',
        focus: true,
      });
    },
    // 处理当前选中值
    dealSelectedValue(item) {
      const { selectedCache } = this.data;
      const index = selectedCache.findIndex((v) => v.value === item.value);
      if (index > -1) {
        return [...selectedCache.slice(0, index), ...selectedCache.slice(index + 1)];
      }

      return [...selectedCache, item];
    },
    // 选中项
    onItemClick: function (e) {
      const item = e.currentTarget.dataset.value;
      if (item.disabled) return;
      if (this.data.multiple) {
        this.setData({
          selectedCache: this.dealSelectedValue(item),
        });
      } else {
        this.triggerEvent('onSelectPicker', item);
      }
    },
    // 滚动到底部
    bindscrolltolower: function () {
      if (this.properties.loadStatus === 0) {
        const pageNo = this.data.pageNo + 1;
        this.setData({ pageNo });
        this.triggerEvent('_childFetchData', {
          pageNo,
          searchValue: this.data.searchValue,
        });
      }
    },
    // 重试
    onRetry: function () {
      const { pageNo } = this.data;
      if (this.properties.loadStatus === 2) {
        this.setData({
          status: 1,
        });
        this.triggerEvent('_childFetchData', {
          pageNo: pageNo,
          searchValue: this.data.searchValue,
        });
      }
    },
  },
  observers: {
    'option,searchValue,filterable': function (option, searchValue, filterable) {
      const { ignoreCase } = this.properties;
      if (filterable) {
        this.setData({
          showOption: option,
        });
      } else {
        const searchRange = option.filter((item) => {
          if (ignoreCase) {
            return String(item.label).toLowerCase().indexOf(searchValue.toLowerCase()) !== -1;
          }
          return item.label.indexOf(searchValue) !== -1;
        });
        this.setData({
          showOption: searchRange,
          status: searchRange.length > 0 ? 0 : 3,
        });
      }
    },
    'selectedValue,showOption': function (selectedValue, showOption) {
      const data = Array.isArray(selectedValue) ? selectedValue : [selectedValue];
      const optionMap = showOption.reduce((pre, cur) => {
        pre[cur.value] = cur;
        return pre;
      }, {});
      const selectedCache = data.map((d) => optionMap[d]).filter((item) => !!item);
      this.setData({ selectedCache });
    },
    loadStatus: function (loadStatus) {
      this.setData({
        status: loadStatus,
      });
    },
    currentPageNo: function (currentPageNo) {
      this.setData({ pageNo: currentPageNo });
    },
  },
});
