import { commonCompBehavior } from '../../utils/common-behavior';
import formFieldBehavior from '../form-field-behavior/form-field-behavior';
import itemBehavior from '../form-field-behavior/item-behavior';
import isEqual from '../../utils/deepEqual';
import { convertSingleValue } from '../../utils/platform';
import { upload } from '../form/uploaderFile/upload';

Component({
  options: {
    virtualHost: true,
  },
  behaviors: [itemBehavior, commonCompBehavior, formFieldBehavior],
  properties: {
    classRoot: {
      type: String,
      value: 'upload-image',
    },
    isCompressBeforeUpload: {
      type: Boolean,
      value: false,
    },
    compressQuality: {
      type: Number,
      value: 0.7,
    },

    compressedHeight: {
      type: Number,
      value: 0,
    },
    compressedWidth: {
      type: Number,
      value: 0,
    },
    sourceType: {
      type: String,
      value: 'both',
    },
    callbacks: {
      type: Object,
      value: null,
    },
    template: {
      type: String,
      value: 'normal',
    },
  },
  data: {
    files: [],
    uploadInstance: {},
    config: { count: 9, maxDuration: 60 },
  },
  methods: {
    initValue: function () {
      const { value, single } = this.properties;
      const v = convertSingleValue(value, single);
      if (!isEqual(v, value)) {
        this.setData({ value: v });
      }
    },
    mapPreviewFile: function (files) {
      return files.map((i) => ({
        ...i,
        progress: i.percent,
        loading: i.status !== 'success',
        cloudId: i.cloudId || i.cloudPath,
        tempUrl: i.tempFilePath,
      }));
    },
    initUploadInstance: function ({ action, config }) {
      const { files } = this.data;
      const _this = this;
      let uploadInstance = {
        action,
        config: { ...config },
        previewFile: files,
        updatePreviewFile(files) {
          _this.setData({ files: _this.mapPreviewFile(files) });
        },
        getPreviewFile() {
          const { files } = _this.data;
          return files;
        },
        onComplete(result) {
          const files = _this.mapPreviewFile(result);
          _this.setData({
            files,
          });
          const cloudPathList = files.map((i) => i.cloudId).filter((j) => !!j);
          let _value = cloudPathList;
          if (_this.data.single) {
            _value = cloudPathList[0] ?? '';
          }
          _this.handleChange({ detail: { value: _value } });
        },
        onSuccess(res) {
          _this.triggerEvent('success', {
            value: res.cloudPath,
            file: res,
          });
        },
        onFail(e) {
          _this.triggerEvent('error', e);
        },
        beforeUpload: _this.data.callbacks?.beforeUpload,
      };

      return uploadInstance;
    },
    setConfig: function (_config) {
      const { single, maxUploadCount, config } = this.data;
      const _maxUploadCount = _config.maxUploadCount ?? maxUploadCount;
      // eslint-disable-next-line @typescript-eslint/no-magic-numbers
      const maxCount = single ? 1 : _maxUploadCount;
      const res = { ...config, count: maxCount, ..._config };
      this.setData({ config: res });
    },
    deleteHandle: function ({ cloudId }) {
      const { files, single } = this.data;
      const fileList = files.filter((d) => d.cloudId !== cloudId);
      this.setData({ files: fileList });

      let value = fileList.map((i) => i?.cloudId).filter((j) => !!j);

      value = single ? value[0] || '' : value;

      this.handleChange({ detail: { value: value } });
    },
    setConfigHandle: function ({ config }) {
      this.setConfig(config);
    },
    init: function () {
      const { config } = this.data;
      const uploadInstance = this.initUploadInstance({ action: 'CHOOSE_MEDIA', config });
      this.setData({ uploadInstance });

      this.setReadonlyAttributes &&
        this.setReadonlyAttributes({
          upload: (_uploadInstance = uploadInstance) => {
            upload(_uploadInstance);
          },
          delete: this.deleteHandle.bind(this),
          uploadInstance,
          setConfig: this.setConfigHandle.bind(this),
        });
    },
    updatePreviewFile: function () {
      const { files } = this.data;
      this.setReadonlyAttributes &&
        this.setReadonlyAttributes({
          previewFile: files,
        });
    },
  },
  observers: {
    'name, value, label, required, visible, disabled, readOnly, sourceType': function () {
      this.updateWidgetAPI();
    },
    'disabled,sourceType,maxUploadCount,sizeType,maxSize,single': function (
      disabled,
      sourceType,
      maxUploadCount,
      sizeType,
      maxSize,
      single,
    ) {
      const config = {
        disabled,
        sourceType,
        maxUploadCount,
        sizeType,
        maxSize,
        single,
      };
      this.setConfig(config);
    },
    config: function () {
      this.init();
    },
    files: function () {
      this.updatePreviewFile();
    },
    value: function (value) {
      const _value = [].concat(value).filter((d) => typeof d === 'string' && d !== '');

      const previewFileMap = this.data.files.reduce((acc, obj) => {
        acc[obj?.cloudId] = obj; // 使用 cloudId 作为新对象的 key
        return acc;
      }, {});
      if (_value.some((i) => !previewFileMap[i])) {
        const previewFile = _value?.length
          ? _value.map((i) => ({
              progress: 100,
              loading: false,
              cloudId: i,
              tempUrl: '',
            }))
          : [];
        this.setData({ files: previewFile });
      }
    },
  },
  lifetimes: {
    attached: function () {
      this.initValue();
      const { disabled, sourceType, sizeType, maxSize } = this.data;
      this.setConfig({ disabled, maxDuration: 60, sourceType, sizeType, maxSize });

      this.updateWidgetAPI();
    },
    ready: function () {
      this.triggerEvent('onReady', {
        uploadInstance: this.data.uploadInstance,
      });
    },
  },
});
